/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Text;
using System.Data;
using System.Reflection;
using Mono.Data.SqliteClient;

using System.Collections;

using Gdk;

// Database tables are defined in Database.sql

public class Database
{
	SqliteConnection connection;
	SqliteCommand command;
	string dbVersion;

	public Database (string file)
	{
		bool create = false;
		if (!File.Exists (file)) {
			create = true;
		}

		try {
			StreamReader srVersion = new StreamReader (file+".version");
			dbVersion = srVersion.ReadToEnd ();
			if (dbVersion != null) {
				dbVersion = dbVersion.Trim();
			}
			srVersion.Close();
		}
		catch {
			dbVersion = null;
		}

		string connectionString = "URI=file:"+file;
		connection = new SqliteConnection(connectionString);
		connection.Open();
		command = connection.CreateCommand();

		if (create) {
			Conf.EmptyCache();
			Assembly thisAssembly = Assembly.GetEntryAssembly ();
			Stream stream = thisAssembly.GetManifestResourceStream("Database.sql");
			if (stream != null) {
				StreamReader sr = new StreamReader (stream);
				string sql = sr.ReadToEnd();
				command.CommandText = sql;
				command.ExecuteNonQuery();

				StreamWriter swVersion = new StreamWriter (file+".version", false);
				swVersion.Write (Defines.VERSION);
				swVersion.Close();

				dbVersion = Defines.VERSION;
			}
			else {
				System.Console.WriteLine("Error creating the database");
			}
		}

		if (dbVersion == null || !dbVersion.Equals (Defines.VERSION)) {
			UpdateDatabase(file, dbVersion);
		}
	}

	public void Close ()
	{
		command.Dispose();
		connection.Close();
	}

	private static string SqlString (string s)
	{
		return s.Replace ("'", "''");
	}

	public ArrayList Catalogs ()
	{
		ArrayList array = new ArrayList();

		command.CommandText = "SELECT * FROM catalogs ORDER BY weight";
		IDataReader reader = command.ExecuteReader();
		while(reader.Read()) {
			Catalog catalog = new Catalog (this,
					reader["name"].ToString(),
					reader["table_name"].ToString(),
					reader["short_description"].ToString(),
					reader["long_description"].ToString(),
					reader["image"].ToString(),
					Int32.Parse(reader["weight"].ToString()));
			array.Add (catalog);
		}
		reader.Close();

		return array;
	}

	public ItemList LoadItemList (Catalog catalog)
	{

		ArrayList columns = GetColumns (catalog.Table);
		Hashtable columnsToShow = GetColumnsToShow (catalog.Table);
		ItemList list = new ItemList (catalog, columns, columnsToShow);
		return list;
	}

	public ItemShelf LoadItemShelf (Catalog catalog)
	{
		ItemShelf itemShelf = new ItemShelf (catalog);    	
		return itemShelf;
	}

	public ArrayList GetItems (string table)
	{
		return GetItems (table, null);
	}

	public ArrayList GetItems (string table, string field)
	{
		ArrayList list = new ArrayList ();
		
		string sql = "SELECT * FROM "+table;
		
		if (field!=null)
			sql+=" ORDER BY "+field;

		command.CommandText = sql;
		IDataReader reader = command.ExecuteReader();

		while (reader.Read()) {
			ArrayList columns = new ArrayList();
			for (int i=0; i < reader.FieldCount; i++) {
				if (reader[i] != null) {
					columns.Add (reader[i].ToString());
				}
				else {
					columns.Add (reader[i]);
				}
			}
			list.Add (new Item (table, columns));
		}

		reader.Close();

		return list;
	}

	public ArrayList Search (string searchString, string table)
	{
		return Search (searchString, table, null);
	}

	public ArrayList Search (string searchString, string table, string field)
	{
		StringBuilder sql = new StringBuilder ();
		sql = sql.Append ("SELECT * FROM ");
		sql = sql.Append (table);
		sql = sql.Append (" WHERE ");
		
		ArrayList columnNames = this.GetColumns (table);
		int i = 0;
		foreach (string s in columnNames) {
			if (i++!=0) {
				sql = sql.Append (" OR ");
			}

			sql = sql.Append (s);
			sql = sql.Append (" LIKE '%");
			sql = sql.Append (SqlString(searchString));
			sql = sql.Append ("%'");
		}

		if (field!=null)
			sql = sql.Append (" ORDER BY "+field);

		command.CommandText = sql.ToString();
		IDataReader reader = command.ExecuteReader();

		ArrayList list = new ArrayList ();

		while (reader.Read()) {
			ArrayList columns = new ArrayList();
			for (i=0; i < reader.FieldCount; i++) {
				if (reader[i] != null) {
				columns.Add (reader[i].ToString());
				}
				else {
					columns.Add (null);
				}
			}
			list.Add (new Item (table, columns));
		}

		reader.Close();

		return list;
	}
	
	public ArrayList GetBorrowers ()
	{
		string sql = "SELECT * FROM borrowers";
		command.CommandText = sql;

		SqliteDataReader reader = command.ExecuteReader();
		
		Hashtable names = new Hashtable();
		while (reader.Read()) {
			names.Add (Int32.Parse(reader["id"].ToString()), reader["name"].ToString());
		}
		reader.Close();
		
		ArrayList list = new ArrayList ();
		foreach (int id in names.Keys) {
			sql = "SELECT * FROM lends WHERE borrower="+id;
			command.CommandText = sql;
			reader = command.ExecuteReader();
			
			ArrayList items = new ArrayList ();
			while (reader.Read()) {
				items.Add (Int32.Parse(reader["id"].ToString()));
			}
			list.Add (new Borrower (id, (string)names[id], items));
			reader.Close();
		}
		
		return list;
	}
	
	public Pixbuf GetBorrowerItemPixbuf (int id)
	{
		Pixbuf pix;
		string sql = "SELECT * FROM lends WHERE id="+id;
		command.CommandText = sql;
		SqliteDataReader reader = command.ExecuteReader();
		if (reader.Read()) {
			string table = (string)reader["table_name"];
			reader.Close();
			sql = "SELECT image FROM catalogs WHERE table_name='"+table+"'";
			command.CommandText = sql.ToString();
			reader = command.ExecuteReader();
			if (reader.Read()) {
				pix = Pixbuf.LoadFromResource (reader["image"].ToString());
				reader.Close();
			}
			else {
				reader.Close();
				pix = Pixbuf.LoadFromResource ("empty.png");
			}
		}
		else {
			reader.Close();
			pix = Pixbuf.LoadFromResource ("empty.png");
		}
		
		return pix.ScaleSimple (15, 15, InterpType.Hyper);
	}
	
	public string GetBorrowerItemText (int id)
	{
		string res;
		
		string sql = "SELECT * FROM lends WHERE id="+id;
		command.CommandText = sql;
		SqliteDataReader reader = command.ExecuteReader();
		if (reader.Read()) {
			string table = reader["table_name"].ToString();
			string itemId = reader["item_id"].ToString();
			reader.Close();
			
			sql = "SELECT title FROM "+table+" WHERE id="+itemId;
			
			command.CommandText = sql.ToString();
			reader = command.ExecuteReader();
			if (reader.Read()) {
				res = reader["title"].ToString();
				reader.Close();
			}
			else {
				reader.Close();
				res = null;
			}
		}
		else {
			reader.Close();
			res = null;
		}
		
		return res;
	}
	
	public int LendItem (Item item, Borrower borrower)
	{
		StringBuilder sql = new StringBuilder (200);
		sql = sql.Append ("INSERT INTO lends VALUES (NULL, ");
		sql = sql.Append (borrower.Id);
		sql = sql.Append (", '");
		sql = sql.Append (item.Table);
		sql = sql.Append ("', ");
		sql = sql.Append (item.Id);
		sql = sql.Append (")");
		
		command.CommandText = sql.ToString();
		command.ExecuteNonQuery();

		return ((SqliteCommand)command).LastInsertRowID();
	}
	
	public void ReturnItem (Item item)
	{
		string sql = "DELETE FROM lends WHERE item_id="+item.Id;
		command.CommandText = sql;
		command.ExecuteNonQuery();
	}
	
	public bool IsBorrowed (Item item)
	{
		StringBuilder sql = new StringBuilder (100);
		sql = sql.Append ("SELECT * FROM lends WHERE item_id=");
		sql = sql.Append (item.Id);
		sql = sql.Append (" AND table_name='");
		sql = sql.Append (item.Table);
		sql = sql.Append ("'");
		command.CommandText = sql.ToString();

		SqliteDataReader reader = command.ExecuteReader();
		if (reader.Read()) {
			reader.Close();
			return true;
		}
		else {
			reader.Close();
			return false;
		}
	}

	public int AddBorrower (string name)
	{
		string sql = "INSERT INTO borrowers VALUES (NULL, '"+name+"')";
		command.CommandText = sql;
		command.ExecuteNonQuery();

		return ((SqliteCommand)command).LastInsertRowID();
	}
	
	public ArrayList GetColumns (string table)
	{
		string sql = "SELECT * FROM "+table+"_columns";
		command.CommandText = sql;

		SqliteDataReader reader = command.ExecuteReader();
		if (reader.Read()) {
			ArrayList list = new ArrayList ();
			// Extract the column names for constructing the Item List
			for (int i=0; i<reader.FieldCount; i++) {
				string s = reader.GetName(i);
				if (!s.Equals("aux")) {
					list.Add (s);
				}
			}
			return list;
		}
		else {
			return null;
		}
	}

	public Hashtable GetColumnsToShow (string table)
	{
		string sql = "SELECT * FROM "+table+"_columns";
		command.CommandText = sql;

		IDataReader reader = command.ExecuteReader();
		if (reader.Read()) {
			Hashtable columnsToShow = new Hashtable ();
			int i = 0;
			while (i < reader.FieldCount) {
				string campo = reader.GetName(i).ToString();
				if (!campo.Equals("aux")) {
					columnsToShow.Add (campo, reader[campo].ToString().Equals("Y")?true:false);
				}
				i++;
			}
			return columnsToShow;
		}
		else {
			return null;
		}
	}

	public void SetColumnsToShow (string table, Hashtable hashtable)
	{
		StringBuilder sql = new StringBuilder (1000);
		sql = sql.Append ("BEGIN; ");
		foreach (string s in hashtable.Keys) {
			sql = sql.Append ("UPDATE ");
			sql = sql.Append (table);
			sql = sql.Append ("_columns SET ");
			sql = sql.Append (s);
			sql = sql.Append ("='");
			if ((bool)hashtable[s]) {
				sql = sql.Append ("Y");
			}
			else {
				sql = sql.Append ("N");
			}
			sql = sql.Append ("' WHERE aux='1'; ");
		}
		sql = sql = sql.Append ("COMMIT;");
		command.CommandText = sql.ToString();
		command.ExecuteNonQuery();
	}

	public int AddItem (string table, Item item)
	{
		StringBuilder sql = new StringBuilder (1000);
		sql = sql.Append ("INSERT INTO ");
		sql = sql.Append (table);
		sql = sql.Append (" VALUES (NULL, ");;
		for (int i=1; i < item.Columns.Count; i++) {
			StringBuilder aux = new StringBuilder(100);
			if (item.Columns[i]!=null) {
				aux = aux.Append ("'");
				aux = aux.Append (SqlString(item.Columns[i].ToString()));
				aux = aux.Append ("'");
			}
			else {
				aux = aux.Append ("NULL");
			}
			
			sql = sql.Append (aux);
			
			if (i!=item.Columns.Count-1) {
				sql = sql.Append (", ");
			}
			else {
				sql = sql.Append (")");
			}
		}

		command.CommandText = sql.ToString();
		command.ExecuteNonQuery();

		return ((SqliteCommand)command).LastInsertRowID();
	}

	public void UpdateItem (string table, Item item)
	{
		ArrayList columnNames = this.GetColumns (table);
		StringBuilder sql = new StringBuilder (1000);
		sql = sql.Append ("UPDATE ");
		sql = sql.Append (table);
		sql = sql.Append (" SET ");

		for (int i = 1; i < columnNames.Count; i++) {
			sql = sql.Append (columnNames[i]);
			sql = sql.Append ("=");
			if (item.Columns[i] != null) {
				sql = sql.Append ("'");
				sql = sql.Append (SqlString(item.Columns[i].ToString()));
				sql = sql.Append ("'");
			}
			else {
				sql = sql.Append ("NULL");
			}

			if (i < columnNames.Count - 1) {
				sql = sql.Append (", ");
			}
		}

		sql = sql.Append (" WHERE id=");
		sql = sql.Append (item.Id);

		Console.WriteLine ("----------SQL: "+sql);
		command.CommandText = sql.ToString();
		command.ExecuteNonQuery();
	}

	public void RemoveItem (string table, Item item)
	{
		StringBuilder sql = new StringBuilder (100);
		sql = sql.Append ("DELETE FROM ");
		sql = sql.Append (table);
		sql = sql.Append (" WHERE id=");
		sql = sql.Append (item.Id);
		command.CommandText = sql.ToString();
		command.ExecuteNonQuery();
		Item.RemoveCache (table, item.Id);
	}

	public Item GetItem (string table, int id)
	{
		Item item = null;

		StringBuilder sql = new StringBuilder (100);
		sql = sql.Append ("SELECT * from ");
		sql = sql.Append (table);
		sql = sql.Append (" WHERE id=");
		sql = sql.Append (id);
		command.CommandText = sql.ToString();
		IDataReader reader = command.ExecuteReader();

		if (reader.Read()) {
			ArrayList columns = new ArrayList();
			for (int i=0; i < reader.FieldCount; i++) {
				if (reader[i] != null) {
					columns.Add (reader[i].ToString());
				}
				else {
					columns.Add (null);
				}
			}
			item = new Item (table, columns);
		}

		reader.Close();

		return item;
	}

	private void UpdateDatabase (string file, string dbVersion)
	{
		if (!Defines.VERSION.Equals ("0.0.6")) {
			Assembly thisAssembly = Assembly.GetEntryAssembly ();
			Stream stream = thisAssembly.GetManifestResourceStream("0.0.5.sql");
			if (stream != null) {
				StreamReader sr = new StreamReader (stream);
				string sql = sr.ReadToEnd();
				command.CommandText = sql;
				command.ExecuteNonQuery();
			}
		}

		StreamWriter swVersion = new StreamWriter (file+".version", false);
		swVersion.Write (Defines.VERSION);
		swVersion.Close();
	}
}
