/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Net;
using System.Collections;

using Gdk;
using Gtk;
using Glade;
using Gnome;

using Mono.Posix;

public class GladeApp
{
	private Gnome.App app;
	private Gnome.Program program;
	private Database database;
	private CatalogList catalogList;
	private BorrowerList borrowerList;

	// Lists widgets
	private ScrolledWindow swCatalogs;
	private ScrolledWindow swBorrowers;
	private ScrolledWindow swItems;
	private ScrolledWindow swPresentation;
	private HPaned itemListPaned;
	private HPaned catalogListPaned;
	private VPaned borrowersPaned;

	private Label labelView;

	// Buttons
	MenuItem menuItemAddItem;
	MenuItem menuItemRemoveItem;
	RadioMenuItem menuItemShelfView;
	RadioMenuItem menuItemListView;
	
	Button zoomInButton;
	Button zoomOutButton;
	Button addItemButton;
	Button removeItemButton;

	Gtk.Entry searchEntry;

	public static void Main (string[] args)
	{
		new GladeApp (args);
	}

	public GladeApp (string[] args) 
	{
		program = new Gnome.Program ("mCatalog", "1.0", Gnome.Modules.UI, args);
		Mono.Posix.Catalog.Init ("mcatalog", Defines.GNOME_LOCALE_DIR);

		// Proxy Setup
		string proxy_host = Conf.Get ("/system/http_proxy/host", "");
		int proxy_port = Conf.Get ("/system/http_proxy/port", 8080);
		bool use_proxy = Conf.Get ("/system/http_proxy/use_http_proxy", false);

		if (use_proxy) {
			string proxy = string.Format("http://{0}:{1}/", proxy_host, proxy_port);
			WebProxy proxyObject = new WebProxy(proxy, true);
			System.Net.GlobalProxySelection.Select = proxyObject;
		}

		
		database = new Database (Conf.HomeDir+"/db.db");

		Glade.XML gxml = new Glade.XML (null, "mainwindow.glade", "app1", "mcatalog");
		gxml.Autoconnect (this);
		app = (Gnome.App)gxml["app1"];
		app.DeleteEvent += new DeleteEventHandler (OnWindowDeleteEvent);

		searchEntry = (Gtk.Entry)gxml["entry1"];
		searchEntry.Activated += new EventHandler (OnSearchEntryActivated);

		// Get some widgets needed
		catalogListPaned = (HPaned)gxml["catalogListPaned"];
		itemListPaned = (HPaned)gxml["itemListPaned"];
		borrowersPaned = (VPaned)gxml["borrowersPaned"];
		itemListPaned.SizeRequested += new SizeRequestedHandler (OnItemListPanedResized);

		// Fill the list hbox
		HBox hBoxList = (HBox)gxml["hBoxList"];
		Button buttonList = new Button ();
		buttonList.Clicked += new EventHandler (OnButtonListClicked);
		Gtk.Image image1 = new Gtk.Image (new Gdk.Pixbuf (null, "list.png"));
		image1.Visible = true;
		buttonList.Add (image1);
		buttonList.Relief = ReliefStyle.Half;
		hBoxList.PackStart (buttonList, false, true, 0);
		Button buttonShelf = new Button ();
		buttonShelf.Clicked += new EventHandler (OnButtonShelfClicked);
		Gtk.Image image2 = new Gtk.Image (new Gdk.Pixbuf (null, "shelf.png"));
		image2.Visible = true;
		buttonShelf.Add (image2);
		buttonShelf.Relief = ReliefStyle.Half;
		hBoxList.PackStart (buttonShelf, false, true, 0);
		labelView = new Label("");
		hBoxList.PackStart (labelView, true, true, 4);
		hBoxList.ShowAll();

		// Get the menu items we need to handle
		menuItemAddItem = (MenuItem)gxml["add_item1"];
		menuItemRemoveItem = (MenuItem)gxml["remove_item1"];
		menuItemShelfView = (RadioMenuItem)gxml["shelf1"];
		menuItemListView = (RadioMenuItem)gxml["list1"];
		menuItemAddItem.Sensitive = false;
		menuItemRemoveItem.Sensitive = false;
		menuItemShelfView.Data["view"] = View.Shelf;
		menuItemListView.Data["view"] = View.List;
		if (Conf.ItemView == View.Shelf) {
			menuItemShelfView.Activate();
		}
		else if (Conf.ItemView == View.Shelf) {
			menuItemListView.Activate();
		}

		menuItemShelfView.Toggled += new EventHandler (OnViewToggled);

		// Buttons
		zoomInButton = (Button)gxml["ZoomInButton"];
		zoomOutButton = (Button)gxml["ZoomOutButton"];
		addItemButton = (Button)gxml["AddItemButton"];
		removeItemButton = (Button)gxml["RemoveItemButton"];
		addItemButton.Sensitive = false;
		removeItemButton.Sensitive = false;

		// Populate the catalog tree
		swCatalogs     = (ScrolledWindow)gxml["ScrolledWindowCatalogs"];
		swBorrowers    = (ScrolledWindow)gxml["ScrolledWindowBorrowers"];
		swItems        = (ScrolledWindow)gxml["ScrolledWindowItems"];
		swPresentation = (ScrolledWindow)gxml["ScrolledWindowPresentation"]; 
		PopulateCatalogs ();
		PopulateBorrowers ();

		RestoreWindowState ();

		app.ShowAll();
		program.Run();
	}

	public void PopulateCatalogs ()
	{
		catalogList = new CatalogList(swItems, swPresentation, database);
		catalogList.Selection.Changed += new EventHandler (OnCatalogActivated);
		catalogList.OnItemSelectionChanged += new EventHandler (OnItemSelectionChanged);
		catalogList.OnAddItemRequest += new EventHandler (AddItem);
		catalogList.OnEditItemRequest += new EventHandler (EditItem);
		catalogList.OnLendItemRequest += new EventHandler (LendItem);
		catalogList.OnReturnItemRequest += new EventHandler (ReturnItem);
		catalogList.OnDeleteItemRequest += new EventHandler (DeleteItems);

		int i=0;
		string active = Conf.Get("active_catalog", "books");
		foreach (Catalog catalog in database.Catalogs()) {
			catalogList.AddCatalog (catalog);
			if (catalog.Name.Equals(active)) {
				catalogList.SetSelectedCatalog(i);
			}
			else {
				i++;
			}
		}    
		swCatalogs.Add (catalogList);
		SetInitialView ();
	}
	
	public void PopulateBorrowers ()
	{
		borrowerList = new BorrowerList (catalogList.GetSelectedCatalog(), database);
		swBorrowers.Add (borrowerList);
	}
	
	public void OnPreferencesActivated (object o, EventArgs args)
	{
		PreferencesDialog dialog = new PreferencesDialog (catalogList.ActiveItemShelf);
	}

	// Change the view mode
	public void OnViewToggled (object o, EventArgs args)
	{
		ViewRadioMenuItemChanged((RadioMenuItem)o);
	}

	public void OnButtonShelfClicked (object o, EventArgs args)
	{
		SetView (View.Shelf);
	}

	public void OnButtonListClicked (object o, EventArgs args)
	{
		SetView (View.List);
	}

	public void SetInitialView ()
	{
		View view = Conf.ItemView;
		SetView (view);
	}

	public void ViewRadioMenuItemChanged (RadioMenuItem radio)
	{
		GLib.SList list = radio.Group;
		int i= 0;
		foreach (RadioMenuItem item in list) {
			if (item.Active) {
				SetView ((View)item.Data["view"]);
				break;
			}
		}
	}

	public void SetView (View view)
	{
		Conf.ItemView = view;

		if (view == View.Shelf) {
			zoomInButton.Visible = true;
			zoomOutButton.Visible = true;
			menuItemShelfView.Activate();
		}
		else if (view == View.List) {
			zoomInButton.Visible = false;
			zoomOutButton.Visible = false;
			menuItemListView.Activate();
		}
		catalogList.ActiveView = view;
	}
	
	public void OnZoomInActivated (object o, EventArgs args)
	{
		double zoom = catalogList.ActiveItemShelf.Zoom;
		catalogList.ActiveItemShelf.Zoom = zoom + 0.2;
	}
	
	public void OnZoomOutActivated (object o, EventArgs args)
	{
		double zoom = catalogList.ActiveItemShelf.Zoom;
		catalogList.ActiveItemShelf.Zoom = zoom - 0.2;
	}
	
	public void OnNormalZoomActivated (object o, EventArgs args)
	{
		double zoom = catalogList.ActiveItemShelf.Zoom;
		catalogList.ActiveItemShelf.Zoom = 1;
	}

	// Event fired when a catalog has been selected
	public void OnCatalogActivated (object o, EventArgs args)
	{
		if (catalogList.GetSelectedCatalog() != null) {
			menuItemAddItem.Sensitive = true;
			addItemButton.Sensitive = true;
			labelView.Text = "<b>"+catalogList.GetSelectedCatalog().ShortDescription+"</b>";

			string order = Conf.Get ("ui/shelf_order", "0");
			string sort  = Conf.Get ("ui/shelf_order_sort", "ascending");
			if (!order.Equals ("0")) {
				if (sort.Equals ("ascending")) {
					sort = Mono.Posix.Catalog.GetString ("ascending");
				}
				else {
					sort = Mono.Posix.Catalog.GetString ("descending");
				}
				labelView.Text += String.Format (Mono.Posix.Catalog.GetString (" (Ordered by {0}, {1})"), order, sort);
			}
			
			labelView.UseMarkup = true;
			Conf.Set("active_catalog", catalogList.GetSelectedCatalog().Name);
		}
		else {
			menuItemAddItem.Sensitive = false;
			addItemButton.Sensitive = false;
			labelView.Text = "";
		}
	}

	public void OnItemSelectionChanged (object o, EventArgs args)
	{
		if (catalogList.GetSelectedCatalog().ItemCollection.GetSelectedItems().Count > 0) {
			menuItemRemoveItem.Sensitive = true;
			removeItemButton.Sensitive = true;
		}
		else {
			menuItemRemoveItem.Sensitive = false;
			removeItemButton.Sensitive = false;
		}
	}

	// Add a new item to the active catalog
	public void OnAddItemToolbarButtonClicked ()
	{
		AddItem (null, null);
	}

	public void OnAddItemButtonClicked (object o, EventArgs args)
	{
		AddItem (null, null);
	}

	public void AddItem (object o, EventArgs args)
	{
		catalogList.GetSelectedCatalog().OpenAddItemDialog();
	}

	public void EditItem (object o, EventArgs args)
	{
		catalogList.GetSelectedCatalog().EditSelectedItem();
	}
	
	public void LendItem (object o, EventArgs args)
	{
		catalogList.GetSelectedCatalog().LendSelectedItem();
	}
	
	public void ReturnItem (object o, EventArgs args)
	{
		catalogList.GetSelectedCatalog().ReturnSelectedItem();
	}

	// Delete the active item
	public void OnDeleteItemToolbarButtonClicked ()
	{
		DeleteItems (null, null);
	}

	public void OnDeleteItemButtonClicked (object o, EventArgs args)
	{
		DeleteItems (null, null); 
	}

	public void DeleteItems (object o, EventArgs args)
	{
		Dialog dialog = new MessageDialog (null,
				DialogFlags.Modal,
				MessageType.Question,
				ButtonsType.OkCancel,
				Mono.Posix.Catalog.GetString ("Remove the selected items?"));
		ResponseType response = (ResponseType)dialog.Run();

		if (response == ResponseType.Ok) {
			ArrayList selected = catalogList.GetSelectedCatalog().ItemCollection.GetSelectedItems ();
			foreach (Item item in selected) {
				catalogList.GetSelectedCatalog().RemoveItem (item);
			}
		}

		dialog.Destroy();
	}

	public void OnSearchEntryActivated (object o, EventArgs args)
	{
		catalogList.Search (searchEntry.Text);
	}

	public void OnQuitActivateEvent (object o, EventArgs args)
	{
		Exit ();
	}

	public void OnWindowDeleteEvent (object o, DeleteEventArgs args) 
	{
		Exit ();
		args.RetVal = true;
	}

	private void OnItemListPanedResized (object o, SizeRequestedArgs args)
	{
		Gdk.Rectangle rect = ((Paned)o).Allocation;
		swItems.SetSizeRequest (rect.Width, rect.Height);
	}

	private void RestoreWindowState()
	{
		int width, height;

		width = Conf.Get ("ui/main_window_width", 600);
		height = Conf.Get("ui/main_window_height", 400);

		if (width == Int32.MaxValue) {
			app.Maximize();
		}
		else {
			app.Resize(width, height);
		}

		width = Conf.Get ("ui/catalog_list_width", 130);
		catalogListPaned.Position = width;

		width = Conf.Get ("ui/item_list_width", 360);
		itemListPaned.Position = width;
		
		height = Conf.Get ("ui/borrowers_list_height", 100);
		borrowersPaned.Position = height;
		
		try {
			int zoom = Conf.Get ("ui/shelf_zoom", 1);
			catalogList.ActiveItemShelf.Zoom = zoom;
		}
		catch {}

		app.WindowPosition = WindowPosition.CenterAlways;
	}

	private void SaveWindowState()
	{
		int height, width;
		
		if (app.GdkWindow.State == Gdk.WindowState.Maximized) {
			Conf.Set("ui/main_window_width", Int32.MaxValue);
		}
		else {
			app.GetSize(out width, out height);
			Conf.Set("ui/main_window_width", width);
			Conf.Set("ui/main_window_height", height);
		}

		Conf.Set("ui/catalog_list_width", catalogListPaned.Position);
		Conf.Set("ui/item_list_width", itemListPaned.Position);
		Conf.Set ("ui/borrowers_list_height", borrowersPaned.Position);

		Conf.Sync ();
	}

	public void ShowAboutDialog (System.Object o, EventArgs e)
	{
		string [] authors = new string [] {
			"Cesar Garcia Tapia (tapia@eitig.com)"
		};

		string [] documenters = new string [] {};
		string translaters = Mono.Posix.Catalog.GetString ("translator-credits");
		Pixbuf pixbuf = null;

		Gnome.About about = new Gnome.About ("MCatalog", Defines.VERSION,
				"Copyright (C) 2003, Cesar Garcia Tapia",
				Mono.Posix.Catalog.GetString ("Media cataloger"),
				authors, documenters, translaters, pixbuf);
		about.Show ();
	}

	private void Exit ()
	{
		SaveWindowState();
		database.Close();
		program.Quit();
	}
}
